100 'Monte Carlo simulation - plural scattering model in Turbo Basic v.1.1.
110 'Model for calculation backscatter electron coefficient from bulk samples.
120 'Program was written for IBM PC XT/AT/PS2 computers with min. one
130 'floppy disc and min. 256 kb RAM and Hercules, CGA, EGA or VGA graphic card.
140 'It is also disirable that the computer has a maths co-processor 80x87.
170 'Author:Henrik Kaker, Ob Suhi 23, 2390 Ravne, Slovenia, Henrik.Kaker@guest.arnes.si
180 'Copyright (c) 1988 by Henrik Kaker

200 cls : clear : dim e(51)
210 locate 2
220 color 15, 0, 0
230 title$ = "Monte Carlo Simulation for Bulk Samples"
240 gosub center:
250 color 7, 0, 0
260 title$ = "Copyright (c) 1988 by H. Kaker"
270 gosub center:
280 locate 23
290 color 15, 0, 0
300 title$ = "Use cursor keys to highlight desired options.Then press Enter."
310 gosub center:
320 menopt$(1) = "1. MC Simulation"
330 menopt$(2) = "2. Graphic Card"
340 menopt$(3) = "3. Exit to MS-DOS"
350 last = 3
360 for choice = 1 to last
370 row%(choice) = 2 * choice + 9
380 col%(choice) = 29
390 next
400 gosub menu:
410 on pick goto 500, 7000, 7900

500 'Monte Carlo simulation
510 'Reading data for graphic card
520 gosub kar: : cls
530 'Input data for simulation
540 color 15, 0, 0
550 locate 2
555 title$ = "Monte Carlo Simulation for Bulk Samples"
556 gosub center:
560 color 7, 0, 0
570 locate 8, 1: input "Input beam energy in keV  ", ei
580 locate 10, 1: input "Target atomic number is  ", z
590 locate 12, 1: input "Target atomic weight is  ", a
600 locate 14, 1: input "Target density in g/cm^3 is  ", rh
650 locate 8, 45: input "Tilt angle in degrees is  ", ti
660 locate 10, 45: input "Number of trajectories is  ", n

740 'Calculation Bethe range
750 'Stopping power in volts/microns (modified expression)
760 def fnsp (en) = 78500 * z * (log(1.166 * (en + .85 * jb) / jb)) / (a * en)
770 'Mean ionization potential,Berger-Selzer
780 jb = (9.76 * z + (58.5 / (z ^ .19))) * .001
790 'Screened Rutherford scattering parameter b
800 ah = 0.263* (z ^ .4)
810 b = .072 * z / ah
820 b = b * ((ei / 15) ^ .95)
830 'Calculate range assuming Bethe continuous energy loss by
840 'Simpson's numerical integation method
850 for m = 1 to 21: en = (m - 1) * ei / 20: if en >= (6.34 * jb) then f = 1 / fnsp(en): goto 900
860   f = (sqr(en * jb)) * a / (62400 * z)
900   l = 2: if m / 2 = int(m / 2) then l = 4
920   if m = 1 or m = 21 then l = 1
930   fs = fs + l * f
940 next m
950 'Calculate range and step length for this conditions
960 r = fs * ei / 60
970 ss = r / 50
980 e(1) = ei
990 'Calculate 50 step energy profile for electron beam by
995 '4-th order Runge-Kutta method
1000 for m = 2 to 51
1010  if e(m - 1) < (6.34 * jb) then 1110
1020  a1 = ss * fnsp(e(m - 1))
1030  if a1 / 2 > e(m - 1) then 1110
1040  a2 = ss * fnsp(e(m - 1) - a1 / 2)
1050  if a2 / 2 > e(m - 1) then 1110
1060  a3 = ss * fnsp(e(m - 1) - a2 / 2)
1070  if a3 > e(m - 1) then 1110
1080  a4 = ss * fnsp(e(m - 1) - a3)
1090  e(m) = e(m - 1) - (a1 + 2 * a2 + 2 * a3 + a4) / 6
1100  goto 1140
1110  em = e(m - 1) ^ 1.5 - 93500 * z * ss / (a * sqr(jb))
1120  e(m) = 0
1130  if em > 1E-09 then e(m) = em ^ .6667
1140 next m
1150 e(51) = 0
1160 'Smoothing energy profile
1170 for m = 2 to 50
1180   e(m) = (e(m) + e(m + 1)) / 2
1190 next m
1200 'Bethe range in microns
1210 sl = ss * 500000 / rh
1220 locate 18, 1: print "Bethe range is  ";
1230 print using "###.##"; sl;
1240 print " microns"
1250 'Step length in um
1260 st = sl / 50
1810 color 15, 0, 0
1820 locate 22
1830 title$ = "Press any key to continue."
1840 gosub center:
1850 gosub temp:

2000 'Plot trajectrories on screen
2020 gosub kar:
2025 cls
2030 if kar$ = "her" then screen 2: ' 720x348,b/w
2040 if kar$ = "cga" then screen 2: ' 620x200 ,b/w
2050 if kar$ = "ega" then screen 9: ' 640x350,16 barv
2060 if kar$ = "vga" then screen 11: '640x480,b/w,all PS/2 model's
2090 'Plot line for sample surface
2100 if kar$ = "her" then 2105 else 2110
2105 xmax = 720: ymax = 348: asp = 43921 / 43922: co = 1: goto 2140
2110 if kar$ = "cga" then 2115 else 2120
2115 xmax = 640: ymax = 200: asp = .4167: co = 1: goto 2140
2120 if kar$ = "ega" then 2125 else 2130
2125 xmax = 640: ymax = 350: asp = .775: co = 14: goto 2140
2130 if kar$ = "vga" then 2135
2135 xmax = 640: ymax = 480: asp = .775: co = 14
2140 line (20, 20)-(xmax - 20, 20), co
2145 'Plot electron beam
2150 line (((xmax / 2) + 20 * tan(ti / 57.4)), 0)-((xmax / 2), 20), co
2160 'Micron marker,find aspect ratio
2170 hsc = xmax / (100 * st): 'pixel/micron
2180 vsc = hsc * asp:    'correction for aspect ratio
2210 'Plot micron marker on screen
2220 if hsc <= 300 then 2230 else 2250
2230 line (10, ymax - 45)-(10 + hsc, ymax - 45), co
2240 locate 23, 2
2245 print "1 um": goto 2300
2250 line (10, ymax - 45)-(10 + (.1 * hsc), ymax - 45), co
2260 locate 23, 2
2270 print "0.1 um"

2300 'Monte Carlo loop
2302 'Initial coordinates
2310 bse = 0
2311 num = 0
2312 for i = 1 to n
2313     num = num + 1
2314     locate 23, 28
2315     print "NE: "; num
2316     x = 0
2320     y = 0
2330     z = 0
2340     cx = 0
2350     cy = sin(ti / 57.4)
2360     cz = cos(ti / 57.4)
2380     pset ((xmax / 2), 20)
2390     for k = 1 to 50
2395          if e(k)=7.11 then goto 2784
2400          'Scattering angles
2410          nu = (rnd(1) ^ .5)
2420          nu = ((1 / nu) - 1)
2430          an = nu * b / e(k)
2440          sp = (an + an) / (1 + (an * an))
2450          cp = (1 - (an * an)) / ((an * an) + 1)
2460          'Azumutal scattering angles
2465          r2 = rnd(3)
2490          ga = 6.28318 * r2
2500          if cx = 0 then cx = .0001
2510          m = -(cx / cz)
2520          n = 1 / sqr(1 + (m * m))
2530          'Calculate new coordinates
2540          v1 = n * sp
2550          v2 = n * m * sp
2560          v3 = cos(ga)
2570          v4 = sin(ga)
2580          ca = (cx * cp) + (v1 * v3) + (cy * v2 * v4)
2590          cb = (cy * cp) + (v4 * (cz * v1 - cx * v2))
2600          cc = (cz * cp) + (v2 * v3) - (cy * v1 * v4)
2610          xn = x + st * ca
2620          yn = y + st * cb
2630          zn = z + st * cc
2640          'Test for electron in sample
2650          y2 = xmax / 2 - (yn * hsc)
2660          z2 = 20 + (zn * vsc)
2670          if y2 >= xmax - 1 then y2 = xmax - 1
2680          if y2 <= 20 then y2 = 20
2690          if z2 >= (ymax - 48) then z2 = (ymax - 48)
2695          if z2 <= 20 then 2700 else 2709
2700               if kar$ = "ega" or kar$ = "vga" then 2704 else 2703
2703               line -(y2, z2): goto 2705
2704               line -(y2, z2), 4
2705               bse = bse + 1
2706               locate 23, 48
2707               print "BSE: "; bse
2708               goto 2784
2709          if kar$ = "ega" or kar$ = "vga" then 2711 else 2710
2710          line -(y2, z2): goto 2720
2711          line -(y2, z2), 4
2720          cx = ca
2730          cy = cb
2740          cz = cc
2750          x = xn
2760          y = yn
2770          z = zn
2780     next k
2784 next i
2795 locate 23, 65
2796 print "BSEC: ";
2797 print using "#.###"; bse / num
2890 gosub temp:
2900 cls
2910 screen 0
2920 goto 100

7000 'Graphic card installation
7010 cls
7015 locate 2
7020 color 15, 0, 0
7025 title$ = "Graphic Card Installation"
7030 gosub center:
7035 color 7, 0, 0
7042 locate 23
7044 color 15, 0, 0
7060 title$ = "Use cursor keys to highlight desired options.Then press Enter."
7070 gosub center:
7080 menopt$(1) = "1. Hercules"
7090 menopt$(2) = "2. CGA Card"
7100 menopt$(3) = "3. EGA Card"
7120 menopt$(4) = "4. VGA Card"
7130 menopt$(5) = "5. Main Menu"
7140 last = 5
7150 for choice = 1 to last
7160 row%(choice) = 2 * choice + 7
7170 col%(choice) = 33
7180 next
7190 gosub menu:
7200 on pick goto 7300, 7310, 7320, 7330, 100

7300 'Hercules card
7305 open "o", #2, "c.drv"
7306 h$ = "her": print #2, h$: close #2
7307 goto 7000
7310 'CGA card
7311 open "o", #3, "c.drv"
7312 h$ = "cga": print #3, h$: close #3
7314 goto 7000
7320 'EGA card
7321 open "o", #4, "c.drv"
7322 h$ = "ega": print #4, h$: close #4
7323 goto 7000
7330 'VGA card
7331 open "o", #5, "c.drv"
7332 h$ = "vga": print #5, h$: close #5
7333 goto 7000

7450 temp:
7460 a$ = inkey$: if a$ = "" then 7460
7470 return

7500 menu:
7510 pick = 1: pick$ = ""
7520 'Create menu
7530 for choice = 1 to last
7540 locate row%(choice), col%(choice)
7550 if pick = choice then color 0, 7
7560 print menopt$(choice)
7570 color 7, 0
7580 next
7600 'Collect input
7610 pick$ = inkey$: if pick$ = "" goto 7610
7620 ascii = asc(pick$)
7630 if ascii = 13 goto 7760
7640 if pick$ = "" then pick = pick + 1: if pick > last then pick = 1: goto 7520
7650 if ascii >= 97 and ascii <= 122 then pick$ = chr$(asc(pick$) - 32)
7660 if len(pick$) = 1 then 7720 else 7670
7670 pick$ = right$(pick$, 1)
7680 ascii = asc(pick$)
7690 if ascii = 72 then pick = pick - 1: if pick = 0 then pick = last
7704 if ascii = 75 then pick = pick - 1: if pick = 0 then pick = last
7706 if ascii = 77 then pick = pick + 1: if pick > last then pick = 1
7708 if ascii = 80 then pick = pick + 1: if pick > last then pick = 1
7710 goto 7520
7720 for choice = 1 to last
7730 if pick$ = left$(menopt$(choice), 1) then pick = choice
7740 next
7750 goto 7520
7760 return

7800 center:
7810 col = (80 - len(title$)) / 2
7820 locate , col
7830 print title$
7840 return

7900 'End of program
7910 color 7, 0, 0
7920 cls
7930 end

8000 kar:
8010 open "i", #1, "c.drv"
8020 input #1, kar$
8030 close #1
8040 return


